@@ -14,6 +14,29 @@
 #include "zend_exceptions.h"
 #include "zend_smart_str.h"
 
+#ifndef ZEND_ARG_INFO_WITH_DEFAULT_VALUE
+#define ZEND_ARG_INFO_WITH_DEFAULT_VALUE(pass_by_ref, name, default_value) \
+	ZEND_ARG_INFO(pass_by_ref, name)
+#endif
+#if PHP_VERSION_ID < 70200
+#undef ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX
+#define ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(name, return_reference, required_num_args, class_name, allow_null) \
+	static const zend_internal_arg_info name[] = { \
+	   	{ (const char*)(zend_uintptr_t)(required_num_args), ( #class_name ), 0, return_reference, allow_null, 0 },
+#endif
+
+#ifndef ZEND_BEGIN_ARG_WITH_RETURN_OBJ_INFO_EX
+#define ZEND_BEGIN_ARG_WITH_RETURN_OBJ_INFO_EX(name, return_reference, required_num_args, class_name, allow_null) \
+	ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(name, return_reference, required_num_args, class_name, allow_null)
+#endif
+
+#ifndef ZEND_ARG_TYPE_INFO_WITH_DEFAULT_VALUE
+#define ZEND_ARG_TYPE_INFO_WITH_DEFAULT_VALUE(pass_by_ref, name, type_hint, allow_null, default_value) \
+	ZEND_ARG_TYPE_INFO(pass_by_ref, name, type_hint, allow_null)
+#endif
+
+#include "ast_arginfo.h"
+
 #define ast_throw_exception(exception_ce, ...) \
 	zend_throw_exception_ex(exception_ce, 0, __VA_ARGS__)
 
@@ -28,7 +51,7 @@
 #define AST_CACHE_SLOT_LINENO   &AST_G(cache_slots)[3 * 2]
 #define AST_CACHE_SLOT_CHILDREN &AST_G(cache_slots)[3 * 3]
 
-#define AST_CURRENT_VERSION 70
+#define AST_CURRENT_VERSION 80
 
 /* Additional flags for BINARY_OP */
 #define AST_BINARY_IS_GREATER 256
@@ -64,6 +87,15 @@
 /* Make IS_STATIC follow IS_ITERABLE in php 7.0 */
 #if PHP_VERSION_ID < 80000
 # define IS_STATIC 20
+# define IS_MIXED 21
+/* In PHP 7.0-7.4, PARAM_REF and PARAM_VARIADIC were 1 and 2. */
+# define PARAM_MODIFIER_PUBLIC    (1 << 2)
+# define PARAM_MODIFIER_PROTECTED (1 << 3)
+# define PARAM_MODIFIER_PRIVATE   (1 << 4)
+#else
+# define PARAM_MODIFIER_PUBLIC    ZEND_ACC_PUBLIC
+# define PARAM_MODIFIER_PROTECTED ZEND_ACC_PROTECTED
+# define PARAM_MODIFIER_PRIVATE   ZEND_ACC_PRIVATE
 #endif
 
 /* This contains state of the ast Node creator. */
@@ -106,6 +138,9 @@ static const char *class_flags[] = {
 static const char *param_flags[] = {
 	AST_FLAG(PARAM_REF),
 	AST_FLAG(PARAM_VARIADIC),
+	AST_FLAG(PARAM_MODIFIER_PUBLIC),
+	AST_FLAG(PARAM_MODIFIER_PROTECTED),
+	AST_FLAG(PARAM_MODIFIER_PRIVATE),
 	NULL
 };
 
@@ -122,6 +157,7 @@ static const char *type_flags[] = {
 	AST_FLAG(TYPE_VOID),
 	AST_FLAG(TYPE_ITERABLE),
 	AST_FLAG(TYPE_STATIC),
+	AST_FLAG(TYPE_MIXED),
 	NULL
 };
 
@@ -280,11 +316,14 @@ static const ast_flag_info flag_info[] = {
 	{ ZEND_AST_PROP_DECL, 1, modifier_flags },
 	{ ZEND_AST_PROP_GROUP, 1, modifier_flags },
 	{ ZEND_AST_CLASS_CONST_DECL, 1, visibility_flags },
+	{ ZEND_AST_CLASS_CONST_GROUP, 1, visibility_flags },
 	{ ZEND_AST_TRAIT_ALIAS, 1, modifier_flags },
 	{ ZEND_AST_DIM, 1, dim_flags },
 	{ ZEND_AST_CONDITIONAL, 1, conditional_flags },
 };
 
+// NOTE(tandre) in php 8, to get a writeable pointer to a property, OBJ_PROP_NUM(AST_CACHE_SLOT_PROPNAME) can be used.
+
 static inline void ast_update_property(zval *object, zend_string *name, zval *value, void **cache_slot) {
 #if PHP_VERSION_ID < 80000
 	zval name_zv;
@@ -313,16 +352,15 @@ static zend_ast *get_ast(zend_string *code, zend_arena **ast_arena, char *filena
 	CG(in_compilation) = 1;
 
 	zend_save_lexical_state(&original_lex_state);
-	if (zend_prepare_string_for_scanning(&code_zv, filename) == SUCCESS) {
-		CG(ast) = NULL;
-		CG(ast_arena) = zend_arena_create(1024 * 32);
-		LANG_SCNG(yy_state) = yycINITIAL;
+	zend_prepare_string_for_scanning(&code_zv, filename);
+	CG(ast) = NULL;
+	CG(ast_arena) = zend_arena_create(1024 * 32);
+	LANG_SCNG(yy_state) = yycINITIAL;
 
-		if (zendparse() != 0) {
-			zend_ast_destroy(CG(ast));
-			zend_arena_destroy(CG(ast_arena));
-			CG(ast) = NULL;
-		}
+	if (zendparse() != 0) {
+		zend_ast_destroy(CG(ast));
+		zend_arena_destroy(CG(ast_arena));
+		CG(ast) = NULL;
 	}
 
 	/* restore_lexical_state changes CG(ast) and CG(ast_arena) */
@@ -337,6 +375,7 @@ static zend_ast *get_ast(zend_string *code, zend_arena **ast_arena, char *filena
 	return ast;
 }
 
+/* Returns whether node->attr (i.e. flags) is used by this node kind. Not to be confused with php 8.0's attributes. */
 static inline zend_bool ast_kind_uses_attr(zend_ast_kind kind) {
 	return kind == ZEND_AST_PARAM || kind == ZEND_AST_TYPE || kind == ZEND_AST_TRAIT_ALIAS
 		|| kind == ZEND_AST_UNARY_OP || kind == ZEND_AST_BINARY_OP || kind == ZEND_AST_ASSIGN_OP
@@ -345,6 +384,7 @@ static inline zend_bool ast_kind_uses_attr(zend_ast_kind kind) {
 		|| kind == ZEND_AST_PROP_GROUP
 		|| kind == ZEND_AST_GROUP_USE || kind == ZEND_AST_USE_ELEM
 		|| kind == AST_NAME || kind == AST_CLOSURE_VAR || kind == ZEND_AST_CLASS_CONST_DECL
+		|| kind == ZEND_AST_CLASS_CONST_GROUP
 		|| kind == ZEND_AST_ARRAY || kind == ZEND_AST_DIM || kind == ZEND_AST_CONDITIONAL;
 }
 
@@ -378,6 +418,9 @@ static inline zend_bool ast_is_name(zend_ast *ast, zend_ast *parent, uint32_t i)
 			|| parent->kind == ZEND_AST_NEW || parent->kind == ZEND_AST_STATIC_CALL
 			|| parent->kind == ZEND_AST_CLASS_CONST || parent->kind == ZEND_AST_STATIC_PROP
 			|| parent->kind == ZEND_AST_PROP_GROUP || parent->kind == ZEND_AST_CLASS_NAME
+#if PHP_VERSION_ID >= 80000
+			|| parent->kind == ZEND_AST_ATTRIBUTE
+#endif
 			;
 	}
 
@@ -422,7 +465,7 @@ static inline zend_bool ast_is_type(zend_ast *ast, zend_ast *parent, uint32_t i)
 
 static inline zend_bool ast_is_var_name(zend_ast *ast, zend_ast *parent, uint32_t i) {
 	return (parent->kind == ZEND_AST_STATIC && i == 0)
-		|| (parent->kind == ZEND_AST_CATCH && i == 1);
+		|| (parent->kind == ZEND_AST_CATCH && i == 1 && ast != NULL);
 }
 
 /* Whether this node may need statement list normalization */
@@ -466,6 +509,8 @@ static const builtin_type_info builtin_types[] = {
 	{ZEND_STRL("object"), IS_OBJECT},
 	{ZEND_STRL("null"), IS_NULL},  /* Null and false for php 8.0 union types */
 	{ZEND_STRL("false"), IS_FALSE},
+	// {ZEND_STRL("static"), IS_STATIC},  /* Impossible to be parsed before php 8 */
+	{ZEND_STRL("mixed"), IS_MIXED},
 	{NULL, 0, IS_UNDEF}
 };
 static inline zend_uchar lookup_builtin_type(const zend_string *name) {
@@ -503,7 +548,11 @@ static inline zend_ast_attr ast_assign_op_to_binary_op(zend_ast_attr attr) {
 static inline zend_ast **ast_get_children(zend_ast *ast, uint32_t *count) {
 	if (ast_kind_is_decl(ast->kind)) {
 		zend_ast_decl *decl = (zend_ast_decl *) ast;
+#if PHP_VERSION_ID >= 80000
+		*count = decl->kind == ZEND_AST_CLASS ? 4 : 5;
+#else
 		*count = decl->kind == ZEND_AST_CLASS ? 3 : 4;
+#endif
 		return decl->child;
 	} else if (zend_ast_is_list(ast)) {
 		zend_ast_list *list = zend_ast_get_list(ast);
@@ -580,6 +629,7 @@ static inline void ast_name_to_zval(zend_ast *child, zend_ast *ast, zval *child_
 	if (child->attr == ZEND_NAME_NOT_FQ
 			&& ast_is_type(child, ast, i)
 			&& (type = lookup_builtin_type(zend_ast_get_str(child)))
+			&& (type != IS_MIXED || state->version >= 80)
 	) {
 		/* Convert "int" etc typehints to TYPE nodes */
 		ast_create_virtual_node_ex(
@@ -617,6 +667,36 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 			}
 		}
 
+#if PHP_VERSION_ID >= 80000
+		if (state->version < 80) {
+			switch (ast_kind) {
+				case ZEND_AST_PARAM:
+					if (i >= 3) {
+						/* Skip attributes and doc comment */
+						continue;
+					}
+					break;
+				case ZEND_AST_METHOD:
+				case ZEND_AST_FUNC_DECL:
+				case ZEND_AST_CLOSURE:
+				case ZEND_AST_ARROW_FUNC:
+					if (i == 4) {
+						continue;
+					}
+					break;
+				case ZEND_AST_CLASS:
+					if (i == 3) {
+						continue;
+					}
+					break;
+				case ZEND_AST_PROP_GROUP:
+					if (i == 2) {
+						continue;
+					}
+					break;
+			}
+		}
+#endif
 		/* This AST_CATCH check should occur before ast_is_name() */
 #if PHP_VERSION_ID < 70100
 		if (ast_kind == ZEND_AST_CATCH && i == 0) {
@@ -624,7 +704,7 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 			zval tmp;
 			ast_create_virtual_node(&tmp, AST_NAME, child->attr, child, state);
 			ast_create_virtual_node_ex(
-				&child_zv, ZEND_AST_NAME_LIST, 0, zend_ast_get_lineno(child), state, 1, &tmp);
+					&child_zv, ZEND_AST_NAME_LIST, 0, zend_ast_get_lineno(child), state, 1, &tmp);
 		} else
 #endif
 		if (ast_is_name(child, ast, i)) {
@@ -688,11 +768,35 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 		zval tmp;
 		ZVAL_NULL(&tmp);
 		zend_hash_add_new(ht, AST_STR(str_docComment), &tmp);
+		return;
+	}
+#endif
+#if PHP_VERSION_ID < 80000
+	if (state->version >= 80) {
+		if (ast_kind == ZEND_AST_PARAM) {
+			zval tmp;
+			ZVAL_NULL(&tmp);
+			zend_hash_add_new(ht, AST_STR(str_attributes), &tmp);
+			zend_hash_add_new(ht, AST_STR(str_docComment), &tmp);
+			return;
+		} else if (ast_kind == ZEND_AST_PROP_GROUP) {
+			zval tmp;
+			ZVAL_NULL(&tmp);
+			zend_hash_add_new(ht, AST_STR(str_attributes), &tmp);
+			return;
+		}
 	}
 #endif
 
 	if (ast_kind_is_decl(ast_kind)) {
 		zval id_zval;
+#if PHP_VERSION_ID < 80000
+		if (state->version >= 80) {
+			zval tmp;
+			ZVAL_NULL(&tmp);
+			zend_hash_add_new(ht, AST_STR(str_attributes), &tmp);
+		}
+#endif
 		ZVAL_LONG(&id_zval, state->declIdCounter);
 		state->declIdCounter++;
 		zend_hash_add_new(ht, AST_STR(str___declId), &id_zval);
@@ -750,6 +854,16 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 			ast->kind = ZEND_AST_UNARY_OP;
 			ast->attr = AST_MINUS;
 			break;
+#if PHP_VERSION_ID >= 80000
+		case ZEND_AST_CLASS_CONST_GROUP:
+			// ast->child is [AST_CLASS_CONST_DECL, optional attributes_list]
+			if (state->version < 80) {
+				// Keep class constants as a list of numerically indexed values in php 8
+				ast->child[0]->attr = ast->attr;
+				ast_to_zval(zv, ast->child[0], state);
+				return;
+			}
+#endif
 #if PHP_VERSION_ID >= 70400
 		case ZEND_AST_PROP_GROUP:
 			if (state->version < 70) {
@@ -871,17 +985,36 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 #if PHP_VERSION_ID < 70400
 	if (ast->kind == ZEND_AST_PROP_DECL && state->version >= 70) {
 		zval type_zval;
-		zval prop_group_zval = *zv;
+		zval prop_group_zval;
+		ZVAL_COPY_VALUE(&prop_group_zval, zv);
 		ZVAL_NULL(&type_zval);
 		// For version 70, create an AST_PROP_GROUP wrapping the created AST_PROP_DECL.
-		ast_create_virtual_node_ex(
-				zv, ZEND_AST_PROP_GROUP, ast->attr, zend_ast_get_lineno(ast), state, 2, &type_zval, &prop_group_zval);
+		if (state->version >= 80) {
+			// For version 80, add a null attributes node.
+			ast_create_virtual_node_ex(
+					zv, ZEND_AST_PROP_GROUP, ast->attr, zend_ast_get_lineno(ast), state, 3, &type_zval, &prop_group_zval, &type_zval);
+		} else {
+			ast_create_virtual_node_ex(
+					zv, ZEND_AST_PROP_GROUP, ast->attr, zend_ast_get_lineno(ast), state, 2, &type_zval, &prop_group_zval);
+		}
 		ast_update_property_long(&prop_group_zval, AST_STR(str_flags), 0, AST_CACHE_SLOT_FLAGS);
 	}
 #endif
+#if PHP_VERSION_ID < 80000
+	if (ast->kind == ZEND_AST_CLASS_CONST_DECL && state->version >= 80) {
+		zval const_decl_zval;
+		zval attributes_zval;
+		ZVAL_COPY_VALUE(&const_decl_zval, zv);
+		ZVAL_NULL(&attributes_zval);
+		ast_update_property_long(zv, AST_STR(str_flags), 0, AST_CACHE_SLOT_FLAGS);
+		// For version 80, create an AST_CLASS_CONST_GROUP wrapping the created AST_CLASS_CONST_DECL
+		ast_create_virtual_node_ex(
+				zv, ZEND_AST_CLASS_CONST_GROUP, ast->attr, zend_ast_get_lineno(ast), state, 2, &const_decl_zval, &attributes_zval);
+	}
+#endif
 }
 
-static const zend_long versions[] = {50, 60, 70};
+static const zend_long versions[] = {50, 60, 70, 80};
 static const size_t versions_count = sizeof(versions)/sizeof(versions[0]);
 
 static inline zend_bool ast_version_deprecated(zend_long version) {
@@ -1132,7 +1265,7 @@ PHP_METHOD(ast_Node, __construct) {
 	zend_long lineno;
 	zend_bool kindNull, flagsNull, linenoNull;
 
-	ZEND_PARSE_PARAMETERS_START(0, 4)
+	ZEND_PARSE_PARAMETERS_START_EX(ZEND_PARSE_PARAMS_THROW, 0, 4)
 		Z_PARAM_OPTIONAL
 		Z_PARAM_LONG_EX(kind, kindNull, 1, 0)
 		Z_PARAM_LONG_EX(flags, flagsNull, 1, 0)
@@ -1168,39 +1301,6 @@ PHP_METHOD(ast_Node, __construct) {
 	}
 }
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_parse_file, 0, 0, 2)
-	ZEND_ARG_INFO(0, filename)
-	ZEND_ARG_INFO(0, version)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_parse_code, 0, 0, 2)
-	ZEND_ARG_INFO(0, code)
-	ZEND_ARG_INFO(0, version)
-	ZEND_ARG_INFO(0, filename)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_get_kind_name, 0, 0, 1)
-	ZEND_ARG_INFO(0, kind)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_kind_uses_flags, 0, 0, 1)
-	ZEND_ARG_INFO(0, kind)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_get_metadata, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_get_supported_versions, 0, 0, 0)
-	ZEND_ARG_INFO(0, exclude_deprecated)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_node_construct, 0, 0, 0)
-	ZEND_ARG_INFO(0, kind)
-	ZEND_ARG_INFO(0, flags)
-	ZEND_ARG_ARRAY_INFO(0, children, 1)
-	ZEND_ARG_INFO(0, lineno)
-ZEND_END_ARG_INFO()
-
 const zend_function_entry ast_functions[] = {
 	ZEND_NS_FE("ast", parse_file, arginfo_parse_file)
 	ZEND_NS_FE("ast", parse_code, arginfo_parse_code)
@@ -1212,7 +1312,7 @@ const zend_function_entry ast_functions[] = {
 };
 
 const zend_function_entry ast_node_functions[] = {
-	PHP_ME(ast_Node, __construct, arginfo_node_construct, ZEND_ACC_PUBLIC)
+	PHP_ME(ast_Node, __construct, arginfo_class_Node___construct, ZEND_ACC_PUBLIC)
 	PHP_FE_END
 };
 
@@ -1263,6 +1363,10 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("MODIFIER_ABSTRACT", ZEND_ACC_ABSTRACT);
 	ast_register_flag_constant("MODIFIER_FINAL", ZEND_ACC_FINAL);
 
+	ast_register_flag_constant("PARAM_MODIFIER_PUBLIC", PARAM_MODIFIER_PUBLIC);
+	ast_register_flag_constant("PARAM_MODIFIER_PROTECTED", PARAM_MODIFIER_PROTECTED);
+	ast_register_flag_constant("PARAM_MODIFIER_PRIVATE", PARAM_MODIFIER_PRIVATE);
+
 	ast_register_flag_constant("RETURNS_REF", ZEND_ACC_RETURN_REFERENCE);
 	ast_register_flag_constant("FUNC_RETURNS_REF", ZEND_ACC_RETURN_REFERENCE);
 	ast_register_flag_constant("FUNC_GENERATOR", ZEND_ACC_GENERATOR);
@@ -1291,6 +1395,7 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("TYPE_VOID", IS_VOID);
 	ast_register_flag_constant("TYPE_ITERABLE", IS_ITERABLE);
 	ast_register_flag_constant("TYPE_STATIC", IS_STATIC);
+	ast_register_flag_constant("TYPE_MIXED", IS_MIXED);
 
 	ast_register_flag_constant("UNARY_BOOL_NOT", ZEND_BOOL_NOT);
 	ast_register_flag_constant("UNARY_BITWISE_NOT", ZEND_BW_NOT);
