@@ -25,6 +25,9 @@ const AST_NAME_LIST = 141;
 const AST_TRAIT_ADAPTATIONS = 142;
 const AST_USE = 143;
 const AST_TYPE_UNION = 144;
+const AST_ATTRIBUTE_LIST = 145;
+const AST_ATTRIBUTE_GROUP = 146;
+const AST_MATCH_ARM_LIST = 147;
 const AST_NAME = 2048;
 const AST_CLOSURE_VAR = 2049;
 const AST_NULLABLE_TYPE = 2050;
@@ -64,44 +67,51 @@ const AST_GOTO = 285;
 const AST_BREAK = 286;
 const AST_CONTINUE = 287;
 const AST_CLASS_NAME = 276;
+const AST_CLASS_CONST_GROUP = 546;
 const AST_DIM = 512;
 const AST_PROP = 513;
-const AST_STATIC_PROP = 514;
-const AST_CALL = 515;
-const AST_CLASS_CONST = 516;
-const AST_ASSIGN = 517;
-const AST_ASSIGN_REF = 518;
-const AST_ASSIGN_OP = 519;
-const AST_BINARY_OP = 520;
-const AST_ARRAY_ELEM = 525;
-const AST_NEW = 526;
-const AST_INSTANCEOF = 527;
-const AST_YIELD = 528;
-const AST_STATIC = 531;
-const AST_WHILE = 532;
-const AST_DO_WHILE = 533;
-const AST_IF_ELEM = 534;
-const AST_SWITCH = 535;
-const AST_SWITCH_CASE = 536;
-const AST_DECLARE = 537;
-const AST_PROP_ELEM = 774;
-const AST_PROP_GROUP = 545;
-const AST_CONST_ELEM = 775;
-const AST_USE_TRAIT = 538;
-const AST_TRAIT_PRECEDENCE = 539;
-const AST_METHOD_REFERENCE = 540;
-const AST_NAMESPACE = 541;
-const AST_USE_ELEM = 542;
-const AST_TRAIT_ALIAS = 543;
-const AST_GROUP_USE = 544;
+const AST_NULLSAFE_PROP = 514;
+const AST_STATIC_PROP = 515;
+const AST_CALL = 516;
+const AST_CLASS_CONST = 517;
+const AST_ASSIGN = 518;
+const AST_ASSIGN_REF = 519;
+const AST_ASSIGN_OP = 520;
+const AST_BINARY_OP = 521;
+const AST_ARRAY_ELEM = 526;
+const AST_NEW = 527;
+const AST_INSTANCEOF = 528;
+const AST_YIELD = 529;
+const AST_STATIC = 532;
+const AST_WHILE = 533;
+const AST_DO_WHILE = 534;
+const AST_IF_ELEM = 535;
+const AST_SWITCH = 536;
+const AST_SWITCH_CASE = 537;
+const AST_DECLARE = 538;
+const AST_PROP_ELEM = 775;
+const AST_PROP_GROUP = 774;
+const AST_CONST_ELEM = 776;
+const AST_USE_TRAIT = 539;
+const AST_TRAIT_PRECEDENCE = 540;
+const AST_METHOD_REFERENCE = 541;
+const AST_NAMESPACE = 542;
+const AST_USE_ELEM = 543;
+const AST_TRAIT_ALIAS = 544;
+const AST_GROUP_USE = 545;
+const AST_ATTRIBUTE = 547;
+const AST_MATCH = 548;
+const AST_MATCH_ARM = 549;
+const AST_NAMED_ARG = 550;
 const AST_METHOD_CALL = 768;
-const AST_STATIC_CALL = 769;
-const AST_CONDITIONAL = 770;
-const AST_TRY = 771;
-const AST_CATCH = 772;
-const AST_PARAM = 773;
+const AST_NULLSAFE_METHOD_CALL = 769;
+const AST_STATIC_CALL = 770;
+const AST_CONDITIONAL = 771;
+const AST_TRY = 772;
+const AST_CATCH = 773;
 const AST_FOR = 1024;
 const AST_FOREACH = 1025;
+const AST_PARAM = 1280;
 // END AST KIND CONSTANTS
 
 // AST FLAG CONSTANTS
@@ -115,6 +125,9 @@ const MODIFIER_PRIVATE = 4;
 const MODIFIER_STATIC = 16;
 const MODIFIER_ABSTRACT = 64;
 const MODIFIER_FINAL = 32;
+const PARAM_MODIFIER_PUBLIC = 1;
+const PARAM_MODIFIER_PROTECTED = 2;
+const PARAM_MODIFIER_PRIVATE = 4;
 const RETURNS_REF = 4096;
 const FUNC_RETURNS_REF = 4096;
 const FUNC_GENERATOR = 16777216;
@@ -125,11 +138,11 @@ const CLASS_FINAL = 32;
 const CLASS_TRAIT = 2;
 const CLASS_INTERFACE = 1;
 const CLASS_ANONYMOUS = 4;
-const PARAM_REF = 1;
-const PARAM_VARIADIC = 2;
+const PARAM_REF = 8;
+const PARAM_VARIADIC = 16;
 const TYPE_NULL = 1;
 const TYPE_FALSE = 2;
-const TYPE_BOOL = 16;
+const TYPE_BOOL = 17;
 const TYPE_LONG = 4;
 const TYPE_DOUBLE = 5;
 const TYPE_STRING = 6;
@@ -139,6 +152,7 @@ const TYPE_CALLABLE = 12;
 const TYPE_VOID = 14;
 const TYPE_ITERABLE = 13;
 const TYPE_STATIC = 15;
+const TYPE_MIXED = 16;
 const UNARY_BOOL_NOT = 14;
 const UNARY_BITWISE_NOT = 13;
 const UNARY_SILENCE = 260;
@@ -177,14 +191,14 @@ const EXEC_REQUIRE_ONCE = 16;
 const USE_NORMAL = 1;
 const USE_FUNCTION = 2;
 const USE_CONST = 4;
-const MAGIC_LINE = 373;
-const MAGIC_FILE = 374;
-const MAGIC_DIR = 375;
-const MAGIC_NAMESPACE = 392;
-const MAGIC_FUNCTION = 379;
-const MAGIC_METHOD = 378;
-const MAGIC_CLASS = 376;
-const MAGIC_TRAIT = 377;
+const MAGIC_LINE = 375;
+const MAGIC_FILE = 376;
+const MAGIC_DIR = 377;
+const MAGIC_NAMESPACE = 382;
+const MAGIC_FUNCTION = 381;
+const MAGIC_METHOD = 380;
+const MAGIC_CLASS = 378;
+const MAGIC_TRAIT = 379;
 const ARRAY_SYNTAX_LIST = 1;
 const ARRAY_SYNTAX_LONG = 2;
 const ARRAY_SYNTAX_SHORT = 3;
@@ -203,7 +217,7 @@ namespace ast;
  *
  * @see https://github.com/nikic/php-ast for version information
  */
-function parse_file($filename, $version)
+function parse_file(string $filename, int $version)
 {
 }
 
@@ -217,7 +231,7 @@ function parse_file($filename, $version)
  *
  * @see https://github.com/nikic/php-ast for version information
  */
-function parse_code($code, $version, $filename = "string code")
+function parse_code(string $code, int $version, string $filename = "string code")
 {
 }
 
@@ -225,7 +239,7 @@ function parse_code($code, $version, $filename = "string code")
  * @param int $kind AST_* constant value defining the kind of an AST node
  * @return string String representation of AST kind value
  */
-function get_kind_name($kind)
+function get_kind_name(int $kind): string
 {
 }
 
@@ -233,7 +247,7 @@ function get_kind_name($kind)
  * @param int $kind AST_* constant value defining the kind of an AST node
  * @return bool Returns true if AST kind uses flags
  */
-function kind_uses_flags($kind)
+function kind_uses_flags(int $kind): bool
 {
 }
 
@@ -244,7 +258,7 @@ function kind_uses_flags($kind)
  *
  * @return Metadata[] Metadata about AST kinds
  */
-function get_metadata()
+function get_metadata(): array
 {
 }
 
@@ -254,7 +268,7 @@ function get_metadata()
  * @param bool $exclude_deprecated Whether to exclude deprecated versions
  * @return int[] Array of supported AST versions
  */
-function get_supported_versions($exclude_deprecated = false)
+function get_supported_versions($exclude_deprecated = false): array
 {
 }
 
